@@ -1,21 +1,20 @@
+# frozen_string_literal: true
 
 module ActionDispatch
-  # Provide callbacks to be executed before and after the request dispatch.
+  # Provides callbacks to be executed before and after dispatching the request.
   class Callbacks
     include ActiveSupport::Callbacks
 
     define_callbacks :call
 
     class << self
-      delegate :to_prepare, :to_cleanup, :to => "ActionDispatch::Reloader"
-    end
+      def before(*args, &block)
+        set_callback(:call, :before, *args, &block)
+      end
 
-    def self.before(*args, &block)
-      set_callback(:call, :before, *args, &block)
-    end
-
-    def self.after(*args, &block)
-      set_callback(:call, :after, *args, &block)
+      def after(*args, &block)
+        set_callback(:call, :after, *args, &block)
+      end
     end
 
     def initialize(app)
