@@ -1,21 +1,24 @@
+# frozen_string_literal: true
+
 require "cases/helper"
-require 'thread'
 
 module ActiveRecord
   module AttributeMethods
     class ReadTest < ActiveRecord::TestCase
-      class FakeColumn < Struct.new(:name)
+      FakeColumn = Struct.new(:name) do
         def type; :integer; end
       end
 
       def setup
-        @klass = Class.new do
+        @klass = Class.new(Class.new { def self.initialize_generated_modules; end }) do
           def self.superclass; Base; end
           def self.base_class; self; end
+          def self.decorate_matching_attribute_types(*); end
 
+          include ActiveRecord::DefineCallbacks
           include ActiveRecord::AttributeMethods
 
-          def self.column_names
+          def self.attribute_names
             %w{ one two three }
           end
 
@@ -23,11 +26,11 @@ module ActiveRecord
           end
 
           def self.columns
-            column_names.map { FakeColumn.new(name) }
+            attribute_names.map { FakeColumn.new(name) }
           end
 
           def self.columns_hash
-            Hash[column_names.map { |name|
+            Hash[attribute_names.map { |name|
               [name, FakeColumn.new(name)]
             }]
           end
@@ -37,21 +40,21 @@ module ActiveRecord
       def test_define_attribute_methods
         instance = @klass.new
 
-        @klass.column_names.each do |name|
-          assert !instance.methods.map(&:to_s).include?(name)
+        @klass.attribute_names.each do |name|
+          assert_not_includes instance.methods.map(&:to_s), name
         end
 
         @klass.define_attribute_methods
 
-        @klass.column_names.each do |name|
-          assert instance.methods.map(&:to_s).include?(name), "#{name} is not defined"
+        @klass.attribute_names.each do |name|
+          assert_includes instance.methods.map(&:to_s), name, "#{name} is not defined"
         end
       end
 
       def test_attribute_methods_generated?
-        assert(!@klass.attribute_methods_generated?, 'attribute_methods_generated?')
+        assert_not @klass.method_defined?(:one)
         @klass.define_attribute_methods
-        assert(@klass.attribute_methods_generated?, 'attribute_methods_generated?')
+        assert @klass.method_defined?(:one)
       end
     end
   end
