@@ -1,9 +1,10 @@
+# frozen_string_literal: true
+
 ActiveRecord::Schema.define do
 
   execute "drop table test_oracle_defaults" rescue nil
   execute "drop sequence test_oracle_defaults_seq" rescue nil
   execute "drop sequence companies_nonstd_seq" rescue nil
-  execute "drop synonym subjects" rescue nil
   execute "drop table defaults" rescue nil
   execute "drop sequence defaults_seq" rescue nil
 
@@ -22,8 +23,6 @@ create sequence test_oracle_defaults_seq minvalue 10000
 
   execute "create sequence companies_nonstd_seq minvalue 10000"
 
-  execute "create synonym subjects for topics"
-
   execute <<-SQL
   CREATE TABLE defaults (
       id integer not null,
@@ -35,10 +34,7 @@ create sequence test_oracle_defaults_seq minvalue 10000
       fixed_time date default TO_DATE('2004-01-01 00:00:00', 'YYYY-MM-DD HH24:MI:SS'),
       char1 varchar2(1) default 'Y',
       char2 varchar2(50) default 'a varchar field',
-      char3 clob default 'a text field',
-      positive_integer integer default 1,
-      negative_integer integer default -1,
-      decimal_number number(3,2) default 2.78
+      char3 clob default 'a text field'
   )
   SQL
   execute "create sequence defaults_seq minvalue 10000"
