@@ -1,24 +1,12 @@
-require 'abstract_unit'
-require 'active_support/core_ext/object/inclusion'
+# frozen_string_literal: true
+
+require "abstract_unit"
+require "active_support/core_ext/object/inclusion"
 
 class InTest < ActiveSupport::TestCase
-  def test_in_multiple_args
-    assert_deprecated do
-      assert :b.in?(:a,:b)
-      assert !:c.in?(:a,:b)
-    end
-  end
-
-  def test_in_multiple_arrays
-    assert_deprecated do
-      assert [1,2].in?([1,2],[2,3])
-      assert ![1,2].in?([1,3],[2,1])
-    end
-  end
-
   def test_in_array
-    assert 1.in?([1,2])
-    assert !3.in?([1,2])
+    assert 1.in?([1, 2])
+    assert !3.in?([1, 2])
   end
 
   def test_in_hash
@@ -39,7 +27,7 @@ class InTest < ActiveSupport::TestCase
   end
 
   def test_in_set
-    s = Set.new([1,2])
+    s = Set.new([1, 2])
     assert 1.in?(s)
     assert !3.in?(s)
   end
@@ -51,14 +39,23 @@ class InTest < ActiveSupport::TestCase
   end
   class C < B
   end
+  class D
+  end
 
   def test_in_module
     assert A.in?(B)
     assert A.in?(C)
     assert !A.in?(A)
+    assert !A.in?(D)
   end
 
   def test_no_method_catching
     assert_raise(ArgumentError) { 1.in?(1) }
   end
+
+  def test_presence_in
+    assert_equal "stuff", "stuff".presence_in(%w( lots of stuff ))
+    assert_nil "stuff".presence_in(%w( lots of crap ))
+    assert_raise(ArgumentError) { 1.presence_in(1) }
+  end
 end
