@@ -31,6 +31,9 @@ class Deprecatee
   def f=(v); end
   deprecate :f=
 
+  deprecate :g
+  def g ;end
+
   module B
     C = 1
   end
@@ -425,6 +428,10 @@ class DeprecationTest < ActiveSupport::TestCase
     end
   end
 
+  def test_deprecate_work_before_define_method
+    assert_deprecated { @dtc.g }
+  end
+
   private
     def deprecator_with_messages
       klass = Class.new(ActiveSupport::Deprecation)
