@@ -1,20 +1,25 @@
+# frozen_string_literal: true
+
 require "active_support/core_ext/module/delegation"
 
 module Rails
   class Application
     class RoutesReloader
       attr_reader :route_sets, :paths
+      attr_accessor :eager_load
       delegate :execute_if_updated, :execute, :updated?, to: :updater
 
       def initialize
         @paths      = []
         @route_sets = []
+        @eager_load = false
       end
 
       def reload!
         clear!
         load_paths
         finalize!
+        route_sets.each(&:eager_load!) if eager_load
       ensure
         revert
       end
@@ -22,11 +27,7 @@ module Rails
     private
 
       def updater
-        @updater ||= begin
-          updater = ActiveSupport::FileUpdateChecker.new(paths) { reload! }
-          updater.execute
-          updater
-        end
+        @updater ||= ActiveSupport::FileUpdateChecker.new(paths) { reload! }
       end
 
       def clear!
@@ -41,9 +42,7 @@ module Rails
       end
 
       def finalize!
-        route_sets.each do |routes|
-          routes.finalize!
-        end
+        route_sets.each(&:finalize!)
       end
 
       def revert
