@@ -1,13 +1,15 @@
-require 'rails/railtie'
-require 'rails/engine/railties'
-require 'active_support/core_ext/module/delegation'
-require 'pathname'
-require 'rbconfig'
+# frozen_string_literal: true
+
+require "rails/railtie"
+require "rails/engine/railties"
+require "active_support/core_ext/module/delegation"
+require "pathname"
+require "thread"
 
 module Rails
   # <tt>Rails::Engine</tt> allows you to wrap a specific Rails application or subset of
   # functionality and share it with other applications or within a larger packaged application.
-  # Since Rails 3.0, every <tt>Rails::Application</tt> is just an engine, which allows for simple
+  # Every <tt>Rails::Application</tt> is just an engine, which allows for simple
   # feature and application sharing.
   #
   # Any <tt>Rails::Engine</tt> is also a <tt>Rails::Railtie</tt>, so the same
@@ -16,10 +18,9 @@ module Rails
   #
   # == Creating an Engine
   #
-  # In Rails versions prior to 3.0, your gems automatically behaved as engines, however,
-  # this coupled Rails to Rubygems. Since Rails 3.0, if you want a gem to automatically
-  # behave as an engine, you have to specify an +Engine+ for it somewhere inside
-  # your plugin's +lib+ folder (similar to how we specify a +Railtie+):
+  # If you want a gem to behave as an engine, you have to specify an +Engine+
+  # for it somewhere inside your plugin's +lib+ folder (similar to how we
+  # specify a +Railtie+):
   #
   #   # lib/my_engine.rb
   #   module MyEngine
@@ -41,7 +42,7 @@ module Rails
   #
   #   class MyEngine < Rails::Engine
   #     # Add a load path for this specific Engine
-  #     config.autoload_paths << File.expand_path("../lib/some/path", __FILE__)
+  #     config.autoload_paths << File.expand_path("lib/some/path", __dir__)
   #
   #     initializer "my_engine.add_middleware" do |app|
   #       app.middleware.use MyEngine::Middleware
@@ -70,10 +71,9 @@ module Rails
   #
   # == Paths
   #
-  # Since Rails 3.0, applications and engines have more flexible path configuration (as
-  # opposed to the previous hardcoded path configuration). This means that you are not
-  # required to place your controllers at <tt>app/controllers</tt>, but in any place
-  # which you find convenient.
+  # Applications and engines have flexible path configuration, meaning that you
+  # are not required to place your controllers at <tt>app/controllers</tt>, but
+  # in any place which you find convenient.
   #
   # For example, let's suppose you want to place your controllers in <tt>lib/controllers</tt>.
   # You can set that as an option:
@@ -111,8 +111,8 @@ module Rails
   #
   # == Endpoint
   #
-  # An engine can be also a rack application. It can be useful if you have a rack application that
-  # you would like to wrap with +Engine+ and provide some of the +Engine+'s features.
+  # An engine can also be a Rack application. It can be useful if you have a Rack application that
+  # you would like to wrap with +Engine+ and provide with some of the +Engine+'s features.
   #
   # To do that, use the +endpoint+ method:
   #
@@ -124,13 +124,13 @@ module Rails
   #
   # Now you can mount your engine in application's routes just like that:
   #
-  #   MyRailsApp::Application.routes.draw do
+  #   Rails.application.routes.draw do
   #     mount MyEngine::Engine => "/engine"
   #   end
   #
   # == Middleware stack
   #
-  # As an engine can now be a rack endpoint, it can also have a middleware
+  # As an engine can now be a Rack endpoint, it can also have a middleware
   # stack. The usage is exactly the same as in <tt>Application</tt>:
   #
   #   module MyEngine
@@ -154,7 +154,7 @@ module Rails
   # Note that now there can be more than one router in your application, and it's better to avoid
   # passing requests through many routers. Consider this situation:
   #
-  #   MyRailsApp::Application.routes.draw do
+  #   Rails.application.routes.draw do
   #     mount MyEngine::Engine => "/blog"
   #     get "/blog/omg" => "main#omg"
   #   end
@@ -164,7 +164,7 @@ module Rails
   # and if there is no such route in +Engine+'s routes, it will be dispatched to <tt>main#omg</tt>.
   # It's much better to swap that:
   #
-  #   MyRailsApp::Application.routes.draw do
+  #   Rails.application.routes.draw do
   #     get "/blog/omg" => "main#omg"
   #     mount MyEngine::Engine => "/blog"
   #   end
@@ -207,42 +207,51 @@ module Rails
   # With such an engine, everything that is inside the +MyEngine+ module will be isolated from
   # the application.
   #
-  # Consider such controller:
+  # Consider this controller:
   #
   #   module MyEngine
   #     class FooController < ActionController::Base
   #     end
   #   end
   #
-  # If an engine is marked as isolated, +FooController+ has access only to helpers from +Engine+ and
-  # <tt>url_helpers</tt> from <tt>MyEngine::Engine.routes</tt>.
+  # If the +MyEngine+ engine is marked as isolated, +FooController+ only has
+  # access to helpers from +MyEngine+, and <tt>url_helpers</tt> from
+  # <tt>MyEngine::Engine.routes</tt>.
   #
-  # The next thing that changes in isolated engines is the behavior of routes. Normally, when you namespace
-  # your controllers, you also need to do namespace all your routes. With an isolated engine,
-  # the namespace is applied by default, so you can ignore it in routes:
+  # The next thing that changes in isolated engines is the behavior of routes.
+  # Normally, when you namespace your controllers, you also need to namespace
+  # the related routes. With an isolated engine, the engine's namespace is
+  # automatically applied, so you don't need to specify it explicitly in your
+  # routes:
   #
   #   MyEngine::Engine.routes.draw do
   #     resources :articles
   #   end
   #
-  # The routes above will automatically point to <tt>MyEngine::ArticlesController</tt>. Furthermore, you don't
-  # need to use longer url helpers like <tt>my_engine_articles_path</tt>. Instead, you should simply use
-  # <tt>articles_path</tt> as you would do with your application.
+  # If +MyEngine+ is isolated, The routes above will point to
+  # <tt>MyEngine::ArticlesController</tt>. You also don't need to use longer
+  # url helpers like +my_engine_articles_path+. Instead, you should simply use
+  # +articles_path+, like you would do with your main application.
   #
-  # To make that behavior consistent with other parts of the framework, an isolated engine also has influence on
-  # <tt>ActiveModel::Naming</tt>. When you use a namespaced model, like <tt>MyEngine::Article</tt>, it will normally
-  # use the prefix "my_engine". In an isolated engine, the prefix will be omitted in url helpers and
-  # form fields for convenience.
+  # To make this behavior consistent with other parts of the framework,
+  # isolated engines also have an effect on <tt>ActiveModel::Naming</tt>. In a
+  # normal Rails app, when you use a namespaced model such as
+  # <tt>Namespace::Article</tt>, <tt>ActiveModel::Naming</tt> will generate
+  # names with the prefix "namespace". In an isolated engine, the prefix will
+  # be omitted in url helpers and form fields, for convenience.
   #
-  #   polymorphic_url(MyEngine::Article.new) # => "articles_path"
+  #   polymorphic_url(MyEngine::Article.new)
+  #   # => "articles_path" # not "my_engine_articles_path"
   #
   #   form_for(MyEngine::Article.new) do
   #     text_field :title # => <input type="text" name="article[title]" id="article_title" />
   #   end
   #
-  # Additionally, an isolated engine will set its name according to namespace, so
-  # MyEngine::Engine.engine_name will be "my_engine". It will also set MyEngine.table_name_prefix
-  # to "my_engine_", changing the MyEngine::Article model to use the my_engine_articles table.
+  # Additionally, an isolated engine will set its own name according to its
+  # namespace, so <tt>MyEngine::Engine.engine_name</tt> will return
+  # "my_engine". It will also set +MyEngine.table_name_prefix+ to "my_engine_",
+  # meaning for example that <tt>MyEngine::Article</tt> will use the
+  # +my_engine_articles+ database table by default.
   #
   # == Using Engine's routes outside Engine
   #
@@ -251,7 +260,7 @@ module Rails
   # created to allow you to do that. Consider such a scenario:
   #
   #   # config/routes.rb
-  #   MyApplication::Application.routes.draw do
+  #   Rails.application.routes.draw do
   #     mount MyEngine::Engine => "/my_engine", as: "my_engine"
   #     get "/foo" => "foo#index"
   #   end
@@ -260,7 +269,7 @@ module Rails
   #
   #   class FooController < ApplicationController
   #     def index
-  #       my_engine.root_url #=> /my_engine/
+  #       my_engine.root_url # => /my_engine/
   #     end
   #   end
   #
@@ -269,7 +278,7 @@ module Rails
   #   module MyEngine
   #     class BarController
   #       def index
-  #         main_app.foo_path #=> /foo
+  #         main_app.foo_path # => /foo
   #       end
   #     end
   #   end
@@ -297,7 +306,7 @@ module Rails
   #     helper MyEngine::SharedEngineHelper
   #   end
   #
-  # If you want to include all of the engine's helpers, you can use #helper method on an engine's
+  # If you want to include all of the engine's helpers, you can use the #helper method on an engine's
   # instance:
   #
   #   class ApplicationController < ActionController::Base
@@ -313,7 +322,7 @@ module Rails
   # Engines can have their own migrations. The default path for migrations is exactly the same
   # as in application: <tt>db/migrate</tt>
   #
-  # To use engine's migrations in application you can use rake task, which copies them to
+  # To use engine's migrations in application you can use the rake task below, which copies them to
   # application's dir:
   #
   #   rake ENGINE_NAME:install:migrations
@@ -329,8 +338,8 @@ module Rails
   #
   # == Loading priority
   #
-  # In order to change engine's priority you can use +config.railties_order+ in main application.
-  # It will affect the priority of loading views, helpers, assets and all the other files
+  # In order to change engine's priority you can use +config.railties_order+ in the main application.
+  # It will affect the priority of loading views, helpers, assets, and all the other files
   # related to engine or application.
   #
   #   # load Blog::Engine with highest priority, followed by application and other railties
@@ -351,8 +360,8 @@ module Rails
           Rails::Railtie::Configuration.eager_load_namespaces << base
 
           base.called_from = begin
-            # Remove the line number from backtraces making sure we don't leave anything behind
-            call_stack = caller.map { |p| p.sub(/:\d+.*/, '') }
+            call_stack = caller_locations.map { |l| l.absolute_path || l.path }
+
             File.dirname(call_stack.detect { |p| p !~ %r[railties[\w.-]*/lib/rails|rack[\w.-]*/lib/rack] })
           end
         end
@@ -360,6 +369,10 @@ module Rails
         super
       end
 
+      def find_root(from)
+        find_root_with_flag "lib", from
+      end
+
       def endpoint(endpoint = nil)
         @endpoint ||= nil
         @endpoint = endpoint if endpoint
@@ -367,9 +380,9 @@ module Rails
       end
 
       def isolate_namespace(mod)
-        engine_name(generate_railtie_name(mod))
+        engine_name(generate_railtie_name(mod.name))
 
-        self.routes.default_scope = { module: ActiveSupport::Inflector.underscore(mod.name) }
+        routes.default_scope = { module: ActiveSupport::Inflector.underscore(mod.name) }
         self.isolated = true
 
         unless mod.respond_to?(:railtie_namespace)
@@ -391,13 +404,13 @@ module Rails
             end
 
             unless mod.respond_to?(:railtie_routes_url_helpers)
-              define_method(:railtie_routes_url_helpers) { railtie.routes.url_helpers }
+              define_method(:railtie_routes_url_helpers) { |include_path_helpers = true| railtie.routes.url_helpers(include_path_helpers) }
             end
           end
         end
       end
 
-      # Finds engine with given path
+      # Finds engine with given path.
       def find(path)
         expanded_path = File.expand_path path
         Rails::Engine.subclasses.each do |klass|
@@ -419,13 +432,13 @@ module Rails
       @env_config          = nil
       @helpers             = nil
       @routes              = nil
+      @app_build_lock      = Mutex.new
       super
     end
 
     # Load console and invoke the registered hooks.
     # Check <tt>Rails::Railtie.console</tt> for more info.
-    def load_console(app=self)
-      require "pp"
+    def load_console(app = self)
       require "rails/console/app"
       require "rails/console/helpers"
       run_console_blocks(app)
@@ -434,22 +447,22 @@ module Rails
 
     # Load Rails runner and invoke the registered hooks.
     # Check <tt>Rails::Railtie.runner</tt> for more info.
-    def load_runner(app=self)
+    def load_runner(app = self)
       run_runner_blocks(app)
       self
     end
 
     # Load Rake, railties tasks and invoke the registered hooks.
     # Check <tt>Rails::Railtie.rake_tasks</tt> for more info.
-    def load_tasks(app=self)
+    def load_tasks(app = self)
       require "rake"
       run_tasks_blocks(app)
       self
     end
 
-    # Load rails generators and invoke the registered hooks.
+    # Load Rails generators and invoke the registered hooks.
     # Check <tt>Rails::Railtie.generators</tt> for more info.
-    def load_generators(app=self)
+    def load_generators(app = self)
       require "rails/generators"
       run_generators_blocks(app)
       Rails::Generators.configure!(app.config.generators)
@@ -460,7 +473,7 @@ module Rails
     # files inside eager_load paths.
     def eager_load!
       config.eager_load_paths.each do |load_path|
-        matcher = /\A#{Regexp.escape(load_path)}\/(.*)\.rb\Z/
+        matcher = /\A#{Regexp.escape(load_path.to_s)}\/(.*)\.rb\Z/
         Dir.glob("#{load_path}/**/*.rb").sort.each do |file|
           require_dependency file.sub(matcher, '\1')
         end
@@ -477,7 +490,7 @@ module Rails
         helpers = Module.new
         all = ActionController::Base.all_helpers_from_path(helpers_paths)
         ActionController::Base.modules_for_helpers(all).each do |mod|
-          helpers.send(:include, mod)
+          helpers.include(mod)
         end
         helpers
       end
@@ -488,12 +501,15 @@ module Rails
       paths["app/helpers"].existent
     end
 
-    # Returns the underlying rack application for this engine.
+    # Returns the underlying Rack application for this engine.
     def app
-      @app ||= begin
-        config.middleware = config.middleware.merge_into(default_middleware_stack)
-        config.middleware.build(endpoint)
-      end
+      @app || @app_build_lock.synchronize {
+        @app ||= begin
+          stack = default_middleware_stack
+          config.middleware = build_middleware.merge_into(stack)
+          config.middleware.build(endpoint)
+        end
+      }
     end
 
     # Returns the endpoint for this engine. If none is registered,
@@ -504,31 +520,26 @@ module Rails
 
     # Define the Rack API for this engine.
     def call(env)
-      env.merge!(env_config)
-      if env['SCRIPT_NAME']
-        env.merge! "ROUTES_#{routes.object_id}_SCRIPT_NAME" => env['SCRIPT_NAME'].dup
-      end
-      app.call(env)
+      req = build_request env
+      app.call req.env
     end
 
     # Defines additional Rack env configuration that is added on each call.
     def env_config
-      @env_config ||= {
-        'action_dispatch.routes' => routes
-      }
+      @env_config ||= {}
     end
 
     # Defines the routes for this engine. If a block is given to
     # routes, it is appended to the engine.
     def routes
-      @routes ||= ActionDispatch::Routing::RouteSet.new
+      @routes ||= ActionDispatch::Routing::RouteSet.new_with_config(config)
       @routes.append(&Proc.new) if block_given?
       @routes
     end
 
     # Define the configuration object for the engine.
     def config
-      @config ||= Engine::Configuration.new(find_root_with_flag("lib"))
+      @config ||= Engine::Configuration.new(self.class.find_root(self.class.called_from))
     end
 
     # Load data from db/seeds.rb file. It can be used in to load engines'
@@ -537,10 +548,10 @@ module Rails
     # Blog::Engine.load_seed
     def load_seed
       seed_file = paths["db/seeds.rb"].existent.first
-      load(seed_file) if seed_file
+      with_inline_jobs { load(seed_file) } if seed_file
     end
 
-    # Add configured load paths to ruby load paths and remove duplicates.
+    # Add configured load paths to Ruby's load path, and remove duplicate entries.
     initializer :set_load_path, before: :bootstrap_hook do
       _all_load_paths.reverse_each do |path|
         $LOAD_PATH.unshift(path) if File.directory?(path)
@@ -552,7 +563,7 @@ module Rails
     # and the load_once paths.
     #
     # This needs to be an initializer, since it needs to run once
-    # per engine and get the engine as a block parameter
+    # per engine and get the engine as a block parameter.
     initializer :set_autoload_paths, before: :bootstrap_hook do
       ActiveSupport::Dependencies.autoload_paths.unshift(*_all_autoload_paths)
       ActiveSupport::Dependencies.autoload_once_paths.unshift(*_all_autoload_once_paths)
@@ -564,10 +575,10 @@ module Rails
     end
 
     initializer :add_routing_paths do |app|
-      paths = self.paths["config/routes.rb"].existent
+      routing_paths = paths["config/routes.rb"].existent
 
-      if routes? || paths.any?
-        app.routes_reloader.paths.unshift(*paths)
+      if routes? || routing_paths.any?
+        app.routes_reloader.paths.unshift(*routing_paths)
         app.routes_reloader.route_sets << routes
       end
     end
@@ -575,14 +586,14 @@ module Rails
     # I18n load paths are a special case since the ones added
     # later have higher priority.
     initializer :add_locales do
-      config.i18n.railties_load_path.concat(paths["config/locales"].existent)
+      config.i18n.railties_load_path << paths["config/locales"]
     end
 
     initializer :add_view_paths do
       views = paths["app/views"].existent
       unless views.empty?
-        ActiveSupport.on_load(:action_controller){ prepend_view_path(views) if respond_to?(:prepend_view_path) }
-        ActiveSupport.on_load(:action_mailer){ prepend_view_path(views) }
+        ActiveSupport.on_load(:action_controller) { prepend_view_path(views) if respond_to?(:prepend_view_path) }
+        ActiveSupport.on_load(:action_mailer) { prepend_view_path(views) }
       end
     end
 
@@ -592,12 +603,6 @@ module Rails
       end
     end
 
-    initializer :append_assets_path, group: :all do |app|
-      app.config.assets.paths.unshift(*paths["vendor/assets"].existent_directories)
-      app.config.assets.paths.unshift(*paths["lib/assets"].existent_directories)
-      app.config.assets.paths.unshift(*paths["app/assets"].existent_directories)
-    end
-
     initializer :prepend_helpers_path do |app|
       if !isolated? || (app == self)
         app.config.helpers_paths.unshift(*paths["app/helpers"].existent)
@@ -606,7 +611,7 @@ module Rails
 
     initializer :load_config_initializers do
       config.paths["config/initializers"].existent.sort.each do |initializer|
-        load(initializer)
+        load_config_initializer(initializer)
       end
     end
 
@@ -616,7 +621,7 @@ module Rails
     end
 
     rake_tasks do
-      next if self.is_a?(Rails::Application)
+      next if is_a?(Rails::Application)
       next unless has_migrations?
 
       namespace railtie_name do
@@ -634,49 +639,79 @@ module Rails
       end
     end
 
-    protected
-
-    def run_tasks_blocks(*) #:nodoc:
-      super
-      paths["lib/tasks"].existent.sort.each { |ext| load(ext) }
-    end
-
     def routes? #:nodoc:
       @routes
     end
 
-    def has_migrations? #:nodoc:
-      paths["db/migrate"].existent.any?
-    end
+    protected
 
-    def find_root_with_flag(flag, default=nil) #:nodoc:
-      root_path = self.class.called_from
-
-      while root_path && File.directory?(root_path) && !File.exist?("#{root_path}/#{flag}")
-        parent = File.dirname(root_path)
-        root_path = parent != root_path && parent
+      def run_tasks_blocks(*) #:nodoc:
+        super
+        paths["lib/tasks"].existent.sort.each { |ext| load(ext) }
       end
 
-      root = File.exist?("#{root_path}/#{flag}") ? root_path : default
-      raise "Could not find root path for #{self}" unless root
+    private
 
-      Pathname.new File.realpath root
-    end
+      def load_config_initializer(initializer) # :doc:
+        ActiveSupport::Notifications.instrument("load_config_initializer.railties", initializer: initializer) do
+          load(initializer)
+        end
+      end
 
-    def default_middleware_stack #:nodoc:
-      ActionDispatch::MiddlewareStack.new
-    end
+      def with_inline_jobs
+        queue_adapter = config.active_job.queue_adapter
+        ActiveSupport.on_load(:active_job) do
+          self.queue_adapter = :inline
+        end
+        yield
+      ensure
+        ActiveSupport.on_load(:active_job) do
+          self.queue_adapter = queue_adapter
+        end
+      end
 
-    def _all_autoload_once_paths #:nodoc:
-      config.autoload_once_paths
-    end
+      def has_migrations?
+        paths["db/migrate"].existent.any?
+      end
 
-    def _all_autoload_paths #:nodoc:
-      @_all_autoload_paths ||= (config.autoload_paths + config.eager_load_paths + config.autoload_once_paths).uniq
-    end
+      def self.find_root_with_flag(flag, root_path, default = nil) #:nodoc:
+        while root_path && File.directory?(root_path) && !File.exist?("#{root_path}/#{flag}")
+          parent = File.dirname(root_path)
+          root_path = parent != root_path && parent
+        end
 
-    def _all_load_paths #:nodoc:
-      @_all_load_paths ||= (config.paths.load_paths + _all_autoload_paths).uniq
-    end
+        root = File.exist?("#{root_path}/#{flag}") ? root_path : default
+        raise "Could not find root path for #{self}" unless root
+
+        Pathname.new File.realpath root
+      end
+
+      def default_middleware_stack
+        ActionDispatch::MiddlewareStack.new
+      end
+
+      def _all_autoload_once_paths
+        config.autoload_once_paths
+      end
+
+      def _all_autoload_paths
+        @_all_autoload_paths ||= (config.autoload_paths + config.eager_load_paths + config.autoload_once_paths).uniq
+      end
+
+      def _all_load_paths
+        @_all_load_paths ||= (config.paths.load_paths + _all_autoload_paths).uniq
+      end
+
+      def build_request(env)
+        env.merge!(env_config)
+        req = ActionDispatch::Request.new env
+        req.routes = routes
+        req.engine_script_name = req.script_name
+        req
+      end
+
+      def build_middleware
+        config.middleware
+      end
   end
 end
