@@ -1,5 +1,7 @@
-require 'rails/generators/erb'
-require 'rails/generators/resource_helpers'
+# frozen_string_literal: true
+
+require "rails/generators/erb"
+require "rails/generators/resource_helpers"
 
 module Erb # :nodoc:
   module Generators # :nodoc:
@@ -14,12 +16,14 @@ module Erb # :nodoc:
 
       def copy_view_files
         available_views.each do |view|
-          filename = filename_with_extensions(view)
-          template filename, File.join("app/views", controller_file_path, filename)
+          formats.each do |format|
+            filename = filename_with_extensions(view, format)
+            template filename, File.join("app/views", controller_file_path, filename)
+          end
         end
       end
 
-    protected
+    private
 
       def available_views
         %w(index edit show new _form)
