@@ -1,7 +1,10 @@
-require 'active_support/testing/autorun'
-require 'rails/configuration'
-require 'active_support/test_case'
-require 'minitest/mock'
+# frozen_string_literal: true
+
+require "active_support"
+require "active_support/testing/autorun"
+require "rails/configuration"
+require "active_support/test_case"
+require "minitest/mock"
 
 module Rails
   module Configuration
@@ -39,7 +42,7 @@ module Rails
         @stack.swap :foo
         @stack.delete :foo
 
-        mock = MiniTest::Mock.new
+        mock = Minitest::Mock.new
         mock.expect :send, nil, [:swap, :foo]
         mock.expect :send, nil, [:delete, :foo]
 
@@ -49,12 +52,12 @@ module Rails
 
       private
 
-      def assert_playback(msg_name, args)
-        mock = MiniTest::Mock.new
-        mock.expect :send, nil, [msg_name, args]
-        @stack.merge_into(mock)
-        mock.verify
-      end
+        def assert_playback(msg_name, args)
+          mock = Minitest::Mock.new
+          mock.expect :send, nil, [msg_name, args]
+          @stack.merge_into(mock)
+          mock.verify
+        end
     end
   end
 end
